!#************************************************************************
!# E.S.O. - 3.60 Upgrade Project
!#
!#
!# who		when		what
!# ----------	--------	------------------------
!# c.levin	920601		created
!# p.leisy	980806		VLT Software adaptation
!# p.leisy	000126 		add the line extraction
!# kbanse	100104 		implement verifications
!# kbanse	100407 		last modif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!#   NAME:	EXTRotate
!# 
!#   SYNOPSIS:	EXTRotate Method Instrument
!#   
!#   DESCRIPTION:	
!#   Computes the Angle Offset to be applied to the Telescope
!#   ???????   It is called from the EFOSClib.tcl trough PCO. 
!#    
!#   FILES:	
!#
!#   ENVIRONMENT:	
!#   PCO must be running together with a background MIDAS session	
!#
!#   RETURN VALUES:
!#   Angle Offset is written in the keyword OUTPUTR(1)
!#
!#------------------------------------------------------------------------
! PARAMETERS:
! P1 --> Image Name
! P2 --> C[URSOR] or G[AUSS] (default)
! P3 --> RILD, REMD, BLMD, DIMD, EFOSC2, NULL
!
! EXTERNAL VARIABLES:
! anglelos, angleril, signrot, anglerot, obsmode
!
defi/par P1 ? C "Name of frame: "
defi/par P2 G C	"Centering Method Cursor or Gauss: "
defi/par P3 "EFO" C	"Observing Mode: [EFO] "
! 
defi/loc cmc/c/1/60	{P1}
defi/loc method/c/1/1   {P2(1:1)}
defi/loc angle/r/1/1	0.
defi/loc dist/r/1/1	0.
defi/loc x1/r/1/1	0.
defi/loc x2/r/1/1	0.
defi/loc y1/r/1/1	0.
defi/loc y2/r/1/1	0.
defi/loc x/r/1/1	0.
defi/loc y/r/1/1	0.
defi/loc x0/r/1/1	0.
defi/loc y0/r/1/1	0.
defi/loc xn/r/1/1	0.
defi/loc yn/r/1/1	0.
defi/loc dx/r/1/1	0.
defi/loc dy/r/1/1	0.
defi/loc xstart/r/1/1	0.
defi/loc ystart/r/1/1	0.
defi/loc xend/r/1/1	0.
defi/loc yend/r/1/1	0.
defi/loc ioff/i/1/2	0,0
defi/loc obsexp/c/1/10	{P3}
defi/loc choix/C/1/1	y
! 
set/overlay
create/table Tangle 15 0 null
! 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get two points from the screen
!
POINTING:
clear/ch ov
set/cur ? c_hair
$xset b 100 2500 200
!$echo "\a"
write/out
write/out "Select the center of 2 objects ..."
write/out
if method .eq. "G" then
   write/out "Method: GAUSS"
   center/gauss {p1},reference Tangle.tbl ? 2,1,2 
   copy/tk Tangle :xcen,:ycen,:xerr,:yerr,:xsig,:ysig,:xfwhm,:yfwhm @1 outputr/r/1/8
   write/keyw rcorr 772.21985,1220.4958,0.10476,0.95027E-01,3.9457,3.3099,9.2913,7.7942
   @@ kcompare rota010:rcorr outputr 1,2 0.001
   @@ kcompare rota012:rcorr outputr 3,4 0.0001
   @@ kcompare rota014:rcorr outputr 5,6 0.001
   @@ kcompare rota016:rcorr outputr 7,8 0.005
   copy/tk Tangle :xcen,:ycen,:xerr,:yerr,:xsig,:ysig,:xfwhm,:yfwhm @2 outputr/r/1/8
   write/keyw rcorr 1583.6344,936.32202,0.80322E-01,0.49040E-01,3.9506,3.2515,9.3029,7.6566,3588.0 
   @@ kcompare rota020:rcorr outputr 1,2 0.001
   @@ kcompare rota022:rcorr outputr 3,4 0.0001
   @@ kcompare rota024:rcorr outputr 5,6 0.001
   @@ kcompare rota026:rcorr outputr 7,8 0.005
 
   x1 = {Tangle,:xcen,1}
   x2 = {Tangle,:xcen,2}
   y1 = {Tangle,:ycen,1}
   y2 = {Tangle,:ycen,2}
else
   write/out "Method: CURSOR"
   get/cursor Tangle ? YN 2,1
   if outputi(1) .lt. 2 then
      write/out
      write/out "Invalid number of coordinates..."
      return
   endif
   if outputi(1) .gt. 2 then
      write/out
      write/out "Too many points selected ..."
      return
   endif
   x1 = {Tangle,:X_coord,1}
   x2 = {Tangle,:X_coord,2}
   y1 = {Tangle,:Y_coord,1}
   y2 = {Tangle,:Y_coord,2}
endif

write/out " "
write/out " x1={x1} y1={y1} x2={x2} y2={y2}"
if x1 .gt. x2 then
    x = x1
    y = y1
    x1 = x2
    y1 = y2
    x2 = x
    y2 = y
endif
! dx always > 0
dx = x2 - x1
dy = y2 - y1
write/out " "
write/out " dx={dx} dy={dy}"
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! draw a line across the points
!
x0 = {{cmc},start(1)}
y0 = {{cmc},start(2)}
xn = {{cmc},start(1)} + {{cmc},step(1)}*({{cmc},npix(1)}-1)
yn = {{cmc},start(2)} + {{cmc},step(2)}*({{cmc},npix(2)}-1)
write/out " "
write/out " x0={x0} y0={y0} xn={xn} yn={yn}"
! 
if dx .eq. 0.0 then
    xstart = x1
    xend = x1
    ystart = y0
    yend = yn
    goto DRAW
else if dy .eq. 0.0 then
    xstart = x0
    xend = xn
    ystart = y1
    yend = y1
    goto DRAW
endif
!
y = y1 + (x0-x1)*dy/dx
x = x1 + (y0-y1)*dx/dy
write/out " "
write/out " x={x} y={y}"
! 
if y .lt. y0 then
    xstart = x
    ystart = y0
elseif y .gt. yn then
    x = x1 + (yn-y1)*dx/dy
    xstart = x
    ystart = yn
else
    xstart = x0
    ystart = y
endif
! 
y = y1 + (xn-x1)*dy/dx
x = x1 + (yn-y1)*dx/dy
write/out " "
write/out " x={x} y={y}"
! 
if y .gt. yn then
    xend = x
    yend = yn
elseif y .le. y0 then
    x = x1 + (y0-y1)*dx/dy
    xend = x
    yend = y0
else
    xend = xn
    yend = y
endif
! 
DRAW:
!
write/out " "
write/out "Draw the line between the 2 points "
write/out "Xstart={xstart} Ystart={ystart} Xend={xend} Yend={yend}"
draw/line {xstart},{ystart},{xend},{yend} F
write/out " "
write/out "Extract the line  "
write/out "{xstart} {ystart} {xend} {yend}"
!EXTRAC/LINE TRACE = {P1} [{xstart},{ystart}:{xend},{yend}]
!SET/GRA pmode=1
!PLOT TRACE
! 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! compute the angle and distance
!
dist = m$sqrt(dx*dx+dy*dy)
if dx .eq. 0 then
    angle = 90
else
    angle = m$atan(dy/dx) 
endif
! 
angle = signrot * (angle + anglerot)
! 
!if obsexp(1:4) .eq. "RILD" then
!   angle = angle + angleril
!else if obsexp(1:4) .ne. "NULL" then ! REMD, BLMD, DIMD
!   angle = angle + anglelos
!endif
!
write/out 
set/format F8.2
write/out "Distance between points (pixels) : {dist}"
write/out "Offset angle to be added to the current rotator offset : {angle}"
write/out
outputr(1) = dist
outputr(2) = angle
write/keyw rcorr 859.73,-19.3
@@ kcompare rota030:rcorr outputr 1,2 0.005

! ****************************
! output for Templates
! ****************************
outputR(1) = {angle}
write/out 
write/out "Final angle = {angle} + Current Rotator Angle "
write/out 

choix = "y"
! INQUIRE/KEY choix " Are you happy with rotation [y],n ?"
IF choix .EQ. "n" THEN
  GOTO POINTING
ENDIF
$xset b 100 2500 100

